@@ -203,6 +203,11 @@ ConfigDlg::~ConfigDlg()
 {
 }
 
+QString ConfigDlg::deviceListText() const
+{
+  return m_dmm->deviceListText();
+}
+
 void
 ConfigDlg::showPage( ConfigDlg::PageType page )
 {
@@ -281,6 +286,8 @@ ConfigDlg::winRect() const
               m_cfg->getInt( "Position", "width", 500 ),
               m_cfg->getInt( "Position", "height", 350 ) );
              
+  //std::cerr << "WRFC: " << rect.x() << " " << rect.y() << " " << rect.width()
+  //    << " " << rect.height() << std::endl;
   return rect;
 }
 
@@ -329,10 +336,6 @@ ConfigDlg::currentTipId() const
 void
 ConfigDlg::applySLOT()
 {
-  m_cfg->setInt( "QtDMM", "version", 0 );
-  m_cfg->setInt( "QtDMM", "revision", 84 );
-  m_cfg->setBool( "QtDMM", "show-tip", showTip() );
-  
   m_cfg->setInt( "Custom colors", "count", QColorDialog::customCount() );
     
   for (int i=0; i<QColorDialog::customCount(); i++)
@@ -342,17 +345,10 @@ ConfigDlg::applySLOT()
     m_cfg->setRGB( "Custom colors", color, QColorDialog::customColor( i ) );
   }
   
-  if (saveWindowPosition())
-  {    
-    m_cfg->setInt( "Position", "x", m_winRect.x() );
-    m_cfg->setInt( "Position", "y", m_winRect.y() );
-    m_cfg->setInt( "Position", "width", m_winRect.width() );
-    m_cfg->setInt( "Position", "height", m_winRect.height() );
-  }
-  else
-  {
-    m_cfg->remove( "Position" );
-  }
+  m_cfg->setInt( "Position", "x", m_winRect.x() );
+  m_cfg->setInt( "Position", "y", m_winRect.y() );
+  m_cfg->setInt( "Position", "width", m_winRect.width() );
+  m_cfg->setInt( "Position", "height", m_winRect.height() );
   
   m_cfg->setInt( "Printer", "page-size", (int)m_printer->pageSize() );
   m_cfg->setInt( "Printer", "page-orientation", (int)m_printer->orientation() );
@@ -423,6 +419,9 @@ ConfigDlg::thresholdChangedSLOT( DMMGraph::CursorMode mode, double value )
   case DMMGraph::Integration:
     m_integration->setThreshold( value );
     break;
+  default:
+    std::cerr << "Unexpected CursorMode in configdlg.cpp:418" << std::endl;
+    break;
   }
 }
 
@@ -549,12 +548,49 @@ ConfigDlg::saveWindowPosition() const
   return m_gui->saveWindowPosition();
 }
 
+bool
+ConfigDlg::saveWindowSize() const
+{
+  return m_gui->saveWindowSize();
+}
+
 void
 ConfigDlg::setShowTipsSLOT( bool on )
 {
   m_gui->setShowTipsSLOT( on );
 }
 
+bool ConfigDlg::showDmmToolbar() const
+{
+  return m_gui->showDmmToolbar();
+}
+
+bool ConfigDlg::showGraphToolbar() const
+{
+  return m_gui->showGraphToolbar();
+}
+
+bool ConfigDlg::showFileToolbar() const
+{
+  return m_gui->showFileToolbar();
+}
+
+bool ConfigDlg::showHelpToolbar() const
+{
+  return m_gui->showHelpToolbar();
+}
+
+bool ConfigDlg::showDisplay() const
+{
+  return m_gui->showDisplay();
+}
+
+void ConfigDlg::setToolbarVisibility( bool disp, bool dmm, bool graph,
+                                      bool file, bool help )
+{
+  m_gui->setToolbarVisibility( disp, dmm, graph, file, help );
+}
+
 /////////////////////////////////////////////////////////////////
 // SCALE
 //
