@@ -15,7 +15,7 @@
 // thereof.  In no event will the author be liable  for any lost revenue
 // or profits or other special, indirect and consequential damages.
 //----------------------------------------------------------------------
-// (c) 2001 Matthias Toussaint
+// (c) 2001-2007 Matthias Toussaint
 //======================================================================
 
 #include <qapplication.h>
@@ -34,11 +34,12 @@
 #include <qscrollbar.h>
 #include <qtextstream.h>
 #include <qtooltip.h>
+#include <qstringlist.h>
 
 #include <dmmgraph.h>
 
 #include <math.h> // RedHat needs it
-
+#include <cstdlib>
 #include <iostream>
 
 #include <icon.xpm>
@@ -55,6 +56,7 @@ DMMGraph::DMMGraph( QWidget *parent, const char *name ) :
   m_sampleTime( 1 ),
   m_sampleLength( 0 ),
   m_running( false ),
+  m_connected( false ),
   m_mode( DMMGraph::Manual ),
   m_mouseDown( false ),
   m_lastValValid( false ),
@@ -102,6 +104,8 @@ DMMGraph::DMMGraph( QWidget *parent, const char *name ) :
   setMouseTracking( true );
   
   m_popup = new QPopupMenu( this );
+  connect( m_popup, SIGNAL( activated(int) ),
+           this, SLOT( popupSLOT(int) ));
 }
 
 DMMGraph::~DMMGraph()
@@ -175,7 +179,7 @@ DMMGraph::print( QPrinter *prt, const QString & title, const QString & comment )
   p.drawText( 0, tRect.height()+10, maxWidth, tHeight, Qt::AlignLeft | Qt::AlignVCenter, 
               tr( "Sampling start:" ) );
   p.drawText( maxWidth+10, tRect.height()+10, w-maxWidth-10, tHeight, Qt::AlignLeft | Qt::AlignVCenter, 
-              m_graphStart.toString() );
+              m_graphStartDateTime.toString() );
   
   p.drawText( 0, tRect.height()+10+tHeight, maxWidth, tHeight, Qt::AlignLeft | Qt::AlignVCenter, 
               tr( "Sampling resolution:" ) );
@@ -318,6 +322,8 @@ void
 DMMGraph::paintVerticalGrid( QPainter *p, double xfactor, double xstep,
                              double maxUnit, double hUnitFact, const QString & hUnit, bool color )
 {
+  int sv = QMAX( 0, scrollbar->value() );
+  
   if (color)
   {
     p->setPen( QPen( m_gridColor, 0, Qt::DotLine ) );
@@ -331,7 +337,7 @@ DMMGraph::paintVerticalGrid( QPainter *p, double xfactor, double xstep,
   double val = xstep;
   while (val < maxUnit)
   {
-    int x = m_graphRect.x() + (int)qRound( 1+(val)/hUnitFact - scrollbar->value()/xfactor );
+    int x = m_graphRect.x() + (int)qRound( 1+(val)/hUnitFact - sv/xfactor );
     if (x > m_graphRect.x() && x < m_graphRect.x()+m_graphRect.width()-1)
     {
       p->drawLine( x, m_graphRect.y(), x, m_graphRect.y()+m_graphRect.height()+3 );
@@ -354,9 +360,11 @@ DMMGraph::paintData( QPainter *p, double xfactor,
 {  
   p->setClipRect( m_graphRect );
   
+  int sv = QMAX( 0, scrollbar->value() );
+
   // draw cursor
   //
-  int x = (int)qRound( (m_pointer-scrollbar->value()-1)/xfactor ) + m_graphRect.x();
+  int x = (int)qRound( (m_pointer-sv-1)/xfactor ) + m_graphRect.x();
   
   if (!printer && x>m_graphRect.x() && x <= m_graphRect.x()+m_graphRect.width()-1)
   {
@@ -373,7 +381,7 @@ DMMGraph::paintData( QPainter *p, double xfactor,
   {
     y = (int)qRound( m_graphRect.y()+
         (m_scaleMax-m_integrationOffset-
-          (*m_arrayInt)[scrollbar->value()]*m_integrationScale)/yfactor );
+          (*m_arrayInt)[sv]*m_integrationScale)/yfactor );
 
     if (color)
     {
@@ -389,9 +397,9 @@ DMMGraph::paintData( QPainter *p, double xfactor,
     pCnt = 1;
     m_drawArray.setPoint( 0, QPoint( m_graphRect.x(), y ) );
 
-    for (int i=scrollbar->value()+1; i<m_pointer; i++)
+    for (int i=sv+1; i<m_pointer; i++)
     {
-      int x = (int)qRound( (i-scrollbar->value())/xfactor ) + 51;
+      int x = (int)qRound( (i-sv)/xfactor ) + 51;
       if (x <= m_graphRect.x()+m_graphRect.width())
       {
         y = (int)qRound( m_graphRect.y()+(m_scaleMax-m_integrationOffset-
@@ -406,7 +414,7 @@ DMMGraph::paintData( QPainter *p, double xfactor,
     
     //y = (int)qRound( 1+(m_scaleMax-(*m_arrayInt)[scrollbar->value()])/yfactor );
     y = (int)qRound( m_graphRect.y()+(m_scaleMax-m_integrationOffset-
-        (*m_arrayInt)[scrollbar->value()]*m_integrationScale)/yfactor );
+        (*m_arrayInt)[sv]*m_integrationScale)/yfactor );
     
     if (color)
     {
@@ -417,9 +425,9 @@ DMMGraph::paintData( QPainter *p, double xfactor,
       p->setPen( Qt::darkGray );
     }
 
-    for (int i=scrollbar->value()+1; i<m_pointer; i++)
+    for (int i=sv+1; i<m_pointer; i++)
     {
-      int x = (int)qRound( (i-scrollbar->value())/xfactor ) + m_graphRect.x();
+      int x = (int)qRound( (i-sv)/xfactor ) + m_graphRect.x();
       if (x <= m_graphRect.x()+m_graphRect.width())
       {
         y = (int)qRound( m_graphRect.y()+(m_scaleMax-m_integrationOffset-
@@ -432,8 +440,8 @@ DMMGraph::paintData( QPainter *p, double xfactor,
   }
   
   // draw data curve
-  //
-  y = (int)qRound( m_graphRect.y()+(m_scaleMax-(*m_array)[scrollbar->value()])/yfactor );
+  //  
+  y = (int)qRound( m_graphRect.y()+(m_scaleMax-(*m_array)[sv])/yfactor );
 
   if (color)
   {
@@ -449,9 +457,9 @@ DMMGraph::paintData( QPainter *p, double xfactor,
   pCnt = 1;
   m_drawArray.setPoint( 0, QPoint( m_graphRect.x(), y ) );
 
-  for (int i=scrollbar->value()+1; i<m_pointer; i++)
+  for (int i=sv+1; i<m_pointer; i++)
   {
-    int x = (int)qRound( (i-scrollbar->value())/xfactor ) + m_graphRect.x();
+    int x = (int)qRound( (i-sv)/xfactor ) + m_graphRect.x();
     if (x <= m_graphRect.x()+m_graphRect.width())
     {
       int y = (int)qRound( m_graphRect.y()+(m_scaleMax-(*m_array)[i])/yfactor );
@@ -462,7 +470,7 @@ DMMGraph::paintData( QPainter *p, double xfactor,
   if (pCnt)
     p->drawPolyline( m_drawArray, 0, pCnt );
   
-  y = (int)qRound( m_graphRect.y()+(m_scaleMax-(*m_array)[scrollbar->value()])/yfactor );
+  y = (int)qRound( m_graphRect.y()+(m_scaleMax-(*m_array)[sv])/yfactor );
 
   if (color)
   {
@@ -473,9 +481,9 @@ DMMGraph::paintData( QPainter *p, double xfactor,
     p->setPen( Qt::black );
   }
   
-  for (int i=scrollbar->value()+1; i<m_pointer; i++)
+  for (int i=sv+1; i<m_pointer; i++)
   {
-    int x = (int)qRound( (i-scrollbar->value())/xfactor ) + m_graphRect.x();
+    int x = (int)qRound( (i-sv)/xfactor ) + m_graphRect.x();
     if (x <= m_graphRect.x()+m_graphRect.width())
     {
       int y = (int)qRound( m_graphRect.y()+(m_scaleMax-(*m_array)[i])/yfactor );
@@ -567,6 +575,7 @@ DMMGraph::setGraphSize( int size, int length )
   m_size = (int)((double)size/m_sampleTime*10.);
   m_length = (int)((double)length/m_sampleTime*10. + 1);
   
+  scrollbar->setMinValue( 0 );
   scrollbar->setMaxValue( m_length-1-m_size );
   scrollbar->setLineStep( (m_size-1)/10 );
   scrollbar->setPageStep( m_size );
@@ -598,7 +607,7 @@ DMMGraph::startSLOT()
   emitInfo();  
   emit running( true );
   
-  m_graphStart = QDateTime::currentDateTime();
+  m_graphStartDateTime = QDateTime::currentDateTime();
   m_externalStarted = false;
   
   (*m_arrayInt)[0] = 0;
@@ -729,11 +738,15 @@ DMMGraph::addValue( double val )
 
     //update();
   }
-
-  emitInfo();
     
   m_sampleCounter++;
-  if (m_sampleCounter == m_sampleTime) m_sampleCounter = 0;
+  m_remainingLength = QMAX( 0, m_remainingLength-1 );
+  
+  if (m_sampleCounter == m_sampleTime) 
+  {
+    m_sampleCounter = 0;
+    emitInfo();
+  }
   
   if (0 == m_remainingLength && m_sampleLength != 0)
   {
@@ -741,8 +754,6 @@ DMMGraph::addValue( double val )
     stopSLOT();
     return;
   }
-  
-  m_remainingLength = QMAX( 0, m_remainingLength-1 );
 }
 
 double
@@ -909,7 +920,7 @@ DMMGraph::clearSLOT()
     }
   }
   
-  m_graphStart = QDateTime::currentDateTime();
+  m_graphStartDateTime = QDateTime::currentDateTime();
   m_first = true;
   m_dirty = false;
   
@@ -919,11 +930,13 @@ DMMGraph::clearSLOT()
 void
 DMMGraph::emitInfo()
 {
-  int w = m_remainingLength / 60 / 60 / 24 / 7;
-  int d = ( m_remainingLength / 60 / 60 / 24 ) % 7;
-  int h = ( m_remainingLength / 60 / 60 ) % (24);
-  int m = (m_remainingLength / 60 ) % 60;
-  int s = m_remainingLength % 60;
+  int seconds = m_remainingLength / 10;
+  
+  int w = seconds / 60 / 60 / 24 / 7;
+  int d = ( seconds / 60 / 60 / 24 ) % 7;
+  int h = ( seconds / 60 / 60 ) % (24);
+  int m = (seconds / 60 ) % 60;
+  int s = seconds % 60;
   
   QString txt;
   
@@ -991,26 +1004,39 @@ DMMGraph::mousePressEvent( QMouseEvent *ev )
   }
   else if(ev->button() == RightButton)
   {
-    /*m_popup->clear();
+    m_popup->clear();
     
-    if (m_running)
+    if (m_connected)
     {
-      m_popup->insertItem( tr("Stop recorder") );
+      m_popup->insertItem( tr("Disconnect"), IDDisconnect );
     }
     else
     {
-      m_popup->insertItem( tr("Start recorder") );
+      m_popup->insertItem( tr("Connect"), IDConnect );
+    }
+    m_popup->insertSeparator();
+    
+    if (m_running)
+    {
+      m_popup->insertItem( tr("Stop recorder"), IDStopRecorder );
+    }
+    else
+    {
+      m_popup->insertItem( tr("Start recorder"), IDStartRecorder );
     }    
-    m_popup->insertItem( tr("Clear graph") );
+    m_popup->insertItem( tr("Clear graph"), IDClearGraph );
     m_popup->insertSeparator();
-    m_popup->insertItem( tr("Automatic scale") );
-    m_popup->insertItem( tr("Include zero line") );
-    m_popup->insertSeparator();
-    m_popup->insertItem( tr("Configure recorder") );
-    m_popup->insertItem( tr("Configure scale") );
-    m_popup->insertItem( tr("Configure graph") );
+    
+    m_popup->insertItem( tr("Configure..."), IDConfigure );
+    
+    if (!m_running)
+    {
+      m_popup->insertSeparator();
+      m_popup->insertItem( tr("Export data..."), IDExportData );
+      m_popup->insertItem( tr("Import data..."), IDImportData );
+    }
 
-    m_popup->popup( ev->globalPos() );*/
+    m_popup->popup( ev->globalPos() );
   }
   else if (ev->button() == MidButton)
   {
@@ -1026,6 +1052,8 @@ DMMGraph::mouseMoveEvent( QMouseEvent *ev )
 {
   if (m_scaleMin == m_scaleMax || m_scaleMin == 1e40) return;
   
+  int sv = QMAX( 0, scrollbar->value() );
+  
   if (!m_mouseDown)
   {
     if (m_mousePan)
@@ -1036,7 +1064,7 @@ DMMGraph::mouseMoveEvent( QMouseEvent *ev )
       //cerr << "delta=" << m_mpos.x()-ev->pos().x() << " offset=" << offset << endl;
       if (fabs(offset) >= 1)
       {
-        scrollbar->setValue( scrollbar->value() + (int)qRound(offset) );
+        scrollbar->setValue( QMIN( scrollbar->maxValue(), sv + (int)qRound(offset) ) );
         m_mpos = ev->pos();
       }
     }
@@ -1139,9 +1167,11 @@ DMMGraph::drawCursor( const QPoint & pos )
   p.setPen( Qt::white );
   p.drawLine( pos.x(), m_graphRect.y(), pos.x(), m_graphRect.height()+m_graphRect.y()-1 );
   
+  int sv = QMAX( 0, scrollbar->value() );
+  
   if (m_crosshair)
   {
-    int x = (int)qRound( (pos.x()-m_graphRect.x())*m_xfactor + scrollbar->value() );
+    int x = (int)qRound( (pos.x()-m_graphRect.x())*m_xfactor + sv );
     if (x < 0) x = 0;
   
     if (x < m_pointer)
@@ -1156,11 +1186,13 @@ DMMGraph::drawCursor( const QPoint & pos )
 void
 DMMGraph::fillInfoBox( const QPoint & pos )
 {
-  int x = (int)qRound( (pos.x()-m_graphRect.x())*m_xfactor + scrollbar->value() );
+  int sv = QMAX( 0, scrollbar->value() );
+  
+  int x = (int)qRound( (pos.x()-m_graphRect.x())*m_xfactor + sv );
   if (x < 0) x = 0;
   int sec = (int)((double)x * m_sampleTime/10.);
   
-  QTime t = m_graphStart.time().addSecs( sec );
+  QTime t = m_graphStartDateTime.time().addSecs( sec );
   QString tmpStr = t.toString();
   
   if (x < m_pointer)
@@ -1180,7 +1212,7 @@ DMMGraph::fillInfoBox( const QPoint & pos )
     if (fabs(val) < 1 && val != 0)
     {
       val *= 1000;
-      prefix = "";
+      prefix = "u";
     }
     if (fabs(val) < 1 && val != 0)
     {
@@ -1222,7 +1254,7 @@ DMMGraph::fillInfoBox( const QPoint & pos )
   m_infoBox->adjustSize();
 }
 
-void
+bool
 DMMGraph::exportDataSLOT()
 {
   QString fn = QFileDialog::getSaveFileName( QString::null, 
@@ -1238,13 +1270,13 @@ DMMGraph::exportDataSLOT()
     
     for (int i=0; i<m_pointer; i++)
     {
-      QDateTime dt = m_graphStart.addSecs( i*(int)qRound(m_sampleTime/10.) );
+      QDateTime dt = m_graphStartDateTime.addSecs( i*(int)qRound(m_sampleTime/10.) );
       QString line;
-      line.sprintf( "%02d.%02d.%04d\t%02d:%02d:%02d\t%g\t%s\n",
+      line.sprintf( "%02d.%02d.%04d\t%02d:%02d:%02d\t%06g\t%s\n",
           dt.date().day(), dt.date().month(), dt.date().year(),
           dt.time().hour(), dt.time().minute(), dt.time().second(),
           (*m_array)[i],
-          m_unit.mid( 1, m_unit.length()-2 ).latin1() );
+          m_unit/*.mid( 1, m_unit.length()-2 )*/.latin1() );
       
       ts << line;
     }
@@ -1252,7 +1284,11 @@ DMMGraph::exportDataSLOT()
     m_dirty = false;
     
     file.close();
+    
+    return true;
   }
+  
+  return false;
 }
 
 void
@@ -1297,6 +1333,10 @@ DMMGraph::importDataSLOT()
     // First pass -> figure out size and sample time              
     QFile file( fn );      
     
+    QStringList token;
+    QStringList dateToken;
+    QStringList timeToken;
+    
     if (file.open( IO_ReadOnly ))
     {
       QTextStream ts( &file );
@@ -1305,78 +1345,95 @@ DMMGraph::importDataSLOT()
 
       if (!line.isNull())
       {
-        /*QRegExp re( "[0-9]+\\.[0-9]+\\.[0-9]+\t[0-9]+:[0-9]+:[0-9]+\t[0-9]*\\.[0-9]+\t.*" );
+        QRegExp re( "[0-9]+\\.[0-9]+\\.[0-9]+\t[0-9]+:[0-9]+:[0-9]+\t[0-9]*\\.[0-9]+\t.*" );
         if (re.match(line) == -1)
         {
           emit error( tr("Oops! Seems not to be a valid file") );
           
           return;
-        }*/
+        }
         
-        QTime startTime = QTime( line.mid( 11, 2 ).toInt(), 
-                                 line.mid( 14, 2 ).toInt(),
-                                 line.mid( 17, 2 ).toInt() );
-        QDate startDate = QDate( line.mid( 6, 4 ).toInt(), 
-                                 line.mid( 3, 2 ).toInt(),
-                                 line.mid( 0, 2 ).toInt() );
+        token = QStringList::split( "\t", line );       
+        dateToken = QStringList::split( ".", token[0] );
+        timeToken = QStringList::split( ":", token[1] );
+        
+        QTime startTime = QTime( timeToken[0].toInt(), 
+                                 timeToken[1].toInt(),
+                                 timeToken[2].toInt() );
+        QDate startDate = QDate( dateToken[2].toInt(), 
+                                 dateToken[1].toInt(),
+                                 dateToken[0].toInt() );
 
-        m_graphStart = QDateTime( startDate, startTime );
+        m_graphStartDateTime = QDateTime( startDate, startTime );
         graphEnd = QDateTime( startDate, startTime );
         
-        setUnit( line.mid( 27, 1 ) );
+        setUnit( line.mid( 27, 3 ) );
         
         cnt++;
 
-        while (!(line = ts.readLine()).isNull())
+        while (!ts.atEnd())
         {
-          QTime nowTime = QTime( line.mid( 11, 2 ).toInt(), 
-                                 line.mid( 14, 2 ).toInt(),
-                                 line.mid( 17, 2 ).toInt() );   
-          QDate nowDate = QDate( line.mid( 6, 4 ).toInt(), 
-                                 line.mid( 3, 2 ).toInt(),
-                                 line.mid( 0, 2 ).toInt() );
-
-          sample += QDateTime( startDate, startTime ).secsTo( 
-              QDateTime( nowDate, nowTime ) );
-
-          startTime = nowTime;
-          startDate = nowDate;
-
-          graphEnd = QDateTime( startDate, startTime );
+          line = ts.readLine();
           
-          cnt++;
-
+          if (!line.isEmpty())
+          {
+            token = QStringList::split( "\t", line );
+            dateToken = QStringList::split( ".", token[0] );
+            timeToken = QStringList::split( ":", token[1] );
+          
+            QTime nowTime = QTime( timeToken[0].toInt(), 
+                                    timeToken[1].toInt(),
+                                    timeToken[2].toInt() );
+            QDate nowDate = QDate( dateToken[2].toInt(), 
+                                    dateToken[1].toInt(),
+                                    dateToken[0].toInt() );
+  
+            sample += QDateTime( startDate, startTime ).secsTo( 
+                QDateTime( nowDate, nowTime ) );
+  
+            startTime = nowTime;
+            startDate = nowDate;
+  
+            graphEnd = QDateTime( startDate, startTime );
+            
+            cnt++;
+          }
         }
       }
       file.close();
     }
     
-    m_sampleTime = m_graphStart.secsTo( graphEnd );
+    //std::cerr << "sample=" << sample << std::endl;
+    
+    m_sampleTime = sample / (cnt>1 ? cnt-1 : 1); //m_graphStartDateTime.secsTo( graphEnd );
     
     int size = m_size*m_sampleTime;
-    int length = (m_length-1)*m_sampleTime;
+    //int length = (m_length-1)*m_sampleTime;
     
     if (cnt > 1)
     {
-      if (sample/(cnt-1) != m_sampleTime)
+      //if (sample/(cnt-1) != m_sampleTime)
       {
         emit sampleTime( m_sampleTime );
       }
       m_sampleTime = sample/(cnt-1);
-    }
+    }        
     
-    if (cnt*m_sampleTime > length)
+  /*  if (cnt*m_sampleTime > length)
     {
       if (size > cnt*m_sampleTime) size = cnt*m_sampleTime;
       emit graphSize( size, cnt*m_sampleTime );
       setGraphSize( size, cnt*m_sampleTime );
-    }
-    
+  }*/
+  
     m_scaleMin =  1e40;
     m_scaleMax = -1e40;
     
     if (file.open( IO_ReadOnly ))
     {
+      // TEST
+      setGraphSize( size, cnt*m_sampleTime );
+      
       int i = 0;
       
       QTextStream ts( &file );
@@ -1384,7 +1441,11 @@ DMMGraph::importDataSLOT()
       
       while (!(line = ts.readLine()).isNull())
       {
-        (*m_array)[i++] = line.mid( 20, 6 ).toDouble();
+        if (!line.isEmpty())
+        {
+          token = QStringList::split( "\t", line );
+          (*m_array)[i++] = token[2].toDouble();
+        }
       }
             
       m_sampleCounter = m_pointer = cnt;
@@ -1399,6 +1460,14 @@ DMMGraph::importDataSLOT()
       
       computeUnitFactor();
     }
+    
+//std::cerr << "min=" << m_scaleMin << " max=" << m_scaleMax << std::endl;
+//std::cerr << "factor=" << m_factor << " prefix=" << 
+//  m_prefix.latin1() << " unit=" << m_unit.latin1() << std::endl;  
+
+//std::cerr << "size=" << size << "cnt*m_sampleTime=" << cnt*m_sampleTime << std::endl;
+    // TEST    
+    emit graphSize( size, cnt*m_sampleTime );
   }
 }
 
@@ -1443,7 +1512,7 @@ DMMGraph::setScale( bool autoScale, bool includeZero, double min, double max )
 
     computeUnitFactor();
   }
-  
+
   resizeEvent( 0 );
 }
 
@@ -1604,7 +1673,7 @@ DMMGraph::computeUnitFactor()
   m_factor = 1.;
   m_prefix = "";
   
-  if (m_unit == "C") return;
+  if (m_unit == "C" || m_unit == "%") return;
   
   if (QMAX( fabs(m_scaleMax*m_factor), fabs(m_scaleMin*m_factor)) > 1000)
   {
@@ -1629,7 +1698,7 @@ DMMGraph::computeUnitFactor()
   if (QMAX( fabs(m_scaleMax*m_factor), fabs(m_scaleMin*m_factor)) < 1)
   {
     m_factor *= 1000.;
-    m_prefix = "";
+    m_prefix = "µ";
   }
   if (QMAX( fabs(m_scaleMax*m_factor), fabs(m_scaleMin*m_factor)) < 1)
   {
@@ -1642,3 +1711,34 @@ DMMGraph::computeUnitFactor()
     m_prefix = "p";
   }
 }
+
+void DMMGraph::popupSLOT( int id )
+{
+  switch (id)
+  {
+  case IDConnect:
+    emit connectDMM( true );
+    break;
+  case IDDisconnect:
+    emit connectDMM( false );
+    break;
+  case IDStopRecorder:
+    stopSLOT();
+    break;
+  case IDStartRecorder:
+    startSLOT();
+    break;
+  case IDClearGraph:
+    clearSLOT();
+    break;
+  case IDConfigure:
+    emit configure();
+    break;
+  case IDExportData:
+    emit exportData();
+    break;
+  case IDImportData:
+    emit importData();
+    break;
+  }
+}
